/*
 * File:   main.c
 * Author: Yui Sakazume
 * This program evaluate IrDA 1.0 SIR software TX/RX
 *
 * Created on 2018/03/24
 */


#include <xc.h>
#include <string.h>
#include <stdio.h>
#include <conio.h>

#pragma config FOSC = INTOSCIO
#pragma config WDTE = OFF
#pragma config PWRTE = ON
#pragma config MCLRE = OFF
#pragma config BOREN = OFF
#pragma config LVP = OFF
#pragma config CPD = OFF
#pragma config WRT = OFF
#pragma config CCPMX = RB3
#pragma config CP = OFF
#define _XTAL_FREQ 8000000

// pin definitions
#define TX_LED  RA0 // Red LED (enabled with low)
#define RX_LED  RA1 // Green LED (enabled with low)
#define IR_TX   RB5   // IR send
#define IR_RX   RB4   // IR receive

// Minimum IRDA pulse is 1.6 usec. Instruction cycle is 500 nsec under 8 MHz clock.
// It requires 3 or 4 cycles. (spends 3 sycles for housekeeping actually)
#define IRDA_PULSE_CYCLES	1

// 416.66 usec with 2400bps.
// need 833 cycles, 208 counts with 1/4 prescaler (spendss 3 sycles for housekeeping actually)
#define	IRDA_BIT_COUNT      205

void init(void){
    TRISA = 0x0;
    PORTA = 0x0;
    TRISB = 0x10;
    PORTB = 0x0;
    ANSEL = 0x0;
    //CMCON = 0x7;
    OPTION_REG = 0x0f; // PSA=WDT, PS=1:4
    INTCON = 0xc0;
    OSCCON=0x70;
    T2CON=0x05; // Timer2 is on, prescale 1/4
    PR2=IRDA_BIT_COUNT;
    __delay_ms(500);
    TX_LED=1;
    RX_LED=1;
}

int getIRDABit(void){
    RBIF=0;
    for(TMR2IF=0; !TMR2IF; );
    return(!RBIF);
}

char getche(void){
    char b=0;
    int i;
    
    for(RBIF=0; !RBIF; );    // detect start bit  
    RX_LED=0;                // margin for standby
    TMR2=10;
    for(TMR2IF=0; !TMR2IF;); // wait end of start bit
    for(i=0; i<8; i++)
        b+=(getIRDABit()<<i);
    if(!getIRDABit())   // stop bit must be 0
        return(0);      // error
    RX_LED=1;
    return(b);
}

void putIRDABit(char b) {
    if (!(b&1)) {			// pulse for 0, no pulse for 1
        IR_TX=0;
        _delay(IRDA_PULSE_CYCLES);  // 1.63 uSec or 1/4 bit clock under IrDA 1.0, actually 2 uSec
        IR_TX=1;
    }
    for(TMR2IF=0; !TMR2IF; );	  // wait for TMR2 expire
}

void putch(unsigned char b) {
    int i;
 
    TMR2IF = 0;
    TMR2 = 3;           // reset TMR2
    TX_LED=0;
    putIRDABit(0);		// start bit
    for(i=0;i<8;i++){
        putIRDABit((b>>i));
    }
    putIRDABit(1);		// stop bit
    TX_LED=1;
}

interrupt void ISR(void){
    if(TMR2IF == 1)
        TMR2IF = 0;
}

void main(void) {
    char str[64];
    init();

    while(1){
        gets(str);
        printf("%s\r\n", str);
    }
}
